@@ -189,11 +189,13 @@ cspi_init_role_table (AccessibleRole *role_table)
   role_table [Accessibility_ROLE_RULER] = SPI_ROLE_RULER;
   role_table [Accessibility_ROLE_APPLICATION] = SPI_ROLE_APPLICATION;
   role_table [Accessibility_ROLE_AUTOCOMPLETE] = SPI_ROLE_AUTOCOMPLETE;
+  role_table [Accessibility_ROLE_EDITBAR] = SPI_ROLE_EDITBAR;
+  role_table [Accessibility_ROLE_EMBEDDED] = SPI_ROLE_EMBEDDED;
 
   return TRUE;
 }
 
-static AccessibleRole
+AccessibleRole
 cspi_role_from_spi_role (Accessibility_Role role)
 {
   /* array is sized according to IDL roles because IDL roles are the index */	
@@ -215,6 +217,21 @@ cspi_role_from_spi_role (Accessibility_Role role)
   return cspi_role; 
 }
 
+AccessibleAttributeSet *
+cspi_attribute_set_from_sequence (const Accessibility_AttributeSet *seq)
+{
+    AccessibleAttributeSet *set = g_new0 (AccessibleAttributeSet, 1);
+    int i;
+
+    set->len = seq->_length;
+    set->attributes = g_newa (char *, set->len);
+    for (i = 0; i < set->len; ++i)
+    {
+	set->attributes[i] = g_strdup (seq->_buffer [i]);
+    }
+    return set;
+}
+
 /**
  * AccessibleRole_getName:
  * @role: an #AccessibleRole object to query.
@@ -427,6 +444,8 @@ Accessible_getRelationSet (Accessible *obj)
 
   cspi_return_val_if_fail (obj != NULL, NULL);
 
+  g_assert (!cspi_exception ());
+
   relation_set =
     Accessibility_Accessible_getRelationSet (CSPI_OBJREF (obj), cspi_ev ());
 
@@ -498,6 +517,32 @@ Accessible_getRoleName (Accessible *obj)
   return retval;
 }
 
+/**
+ * Accessible_getLocalizedRoleName:
+ * @obj: a pointer to the #Accessible object on which to operate.
+ *
+ * Get a UTF-8 string describing the (localized) role this object plays in the UI.
+ * This method will return useful values for roles that fall outside the
+ * enumeration used in Accessible_getRole ().
+ *
+ * Returns: a UTF-8 string specifying the role of this #Accessible object.
+ *
+ **/
+char *
+Accessible_getLocalizedRoleName (Accessible *obj)
+{
+  char *retval;
+
+  cspi_return_val_if_fail (obj != NULL, CORBA_string_dup ("invalid"));
+
+  retval = 
+    Accessibility_Accessible_getLocalizedRoleName (CSPI_OBJREF (obj), cspi_ev ());
+
+  cspi_return_val_if_ev ("getLocalizedRoleName", CORBA_string_dup ("invalid")); 
+
+  return retval;
+}
+
 /**
  * Accessible_getStateSet:
  * @obj: a pointer to the #Accessible object on which to operate.
@@ -530,6 +575,54 @@ Accessible_getStateSet (Accessible *obj)
   return retval;
 }
 
+/**
+ * Accessible_getAttributes:
+ * Get the #AttributeSet representing any assigned 
+ * name-value pair attributes or annotations for this object.
+ * For typographic, textual, or textually-semantic attributes, see
+ * AccessibleText_getAttributes instead.
+ * @obj: The #Accessible being queried.
+ * Returns: The name-value-pair attributes assigned to this object.
+ */
+AccessibleAttributeSet *
+Accessible_getAttributes (Accessible *obj)
+{
+    AccessibleAttributeSet *retval;
+    Accessibility_AttributeSet *corba_seq;
+
+    cspi_return_val_if_fail (obj != NULL, NULL);
+
+    corba_seq = Accessibility_Accessible_getAttributes (
+	CSPI_OBJREF (obj), cspi_ev ());
+    cspi_return_val_if_ev ("getAttributes", NULL);
+    
+    retval = cspi_attribute_set_from_sequence (corba_seq);
+    CORBA_free (corba_seq);
+
+    return NULL;
+}
+
+/**
+ * Accessible_getHostApplication:
+ * Get the containing #AccessibleApplication for an object.
+ * @obj: The #Accessible being queried.
+ * Returns: the containing AccessibleApplication instance for this object.
+ */
+AccessibleApplication *
+Accessible_getHostApplication (Accessible *obj)
+{
+    AccessibleApplication *retval;
+
+    cspi_return_val_if_fail (obj != NULL, NULL);
+
+    retval = Accessible_getApplication (cspi_object_add (
+					    Accessibility_Accessible_getApplication (CSPI_OBJREF (obj),
+										     cspi_ev ())));
+    cspi_return_val_if_ev ("getApplication", NULL);
+
+    return retval;
+}
+
 /* Interface query methods */
 
 /**
@@ -988,7 +1081,7 @@ cspi_init_relation_type_table (AccessibleRelationType *relation_type_table)
   relation_type_table [Accessibility_RELATION_EMBEDS] = SPI_RELATION_EMBEDS;
   relation_type_table [Accessibility_RELATION_EMBEDDED_BY] = SPI_RELATION_EMBEDDED_BY;
   relation_type_table [Accessibility_RELATION_POPUP_FOR] = SPI_RELATION_POPUP_FOR;
-  relation_type_table [Accessibility_RELATION_LAST_DEFINED] = SPI_RELATION_LAST_DEFINED;
+  return TRUE;
 }
 
 static AccessibleRelationType
